using Adaptation.Eaf.Management.ConfigurationData.CellAutomation;
using Adaptation.Ifx.Eaf.EquipmentConnector.File.Configuration;
using Adaptation.Shared;
using Adaptation.Shared.Duplicator;
using Adaptation.Shared.Methods;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Text.Json;
using System.Text.RegularExpressions;

namespace Adaptation.FileHandlers.Kanban;

public class FileRead : Shared.FileRead, IFileRead
{

    private readonly ReadOnlyCollection<string> _CSSLines;
    private readonly ReadOnlyCollection<string> _FrontMatterLines;

    public FileRead(ISMTP smtp, Dictionary<string, string> fileParameter, string cellInstanceName, int? connectionCount, string cellInstanceConnectionName, FileConnectorConfiguration fileConnectorConfiguration, string equipmentTypeName, string parameterizedModelObjectDefinitionType, IList<ModelObjectParameterDefinition> modelObjectParameters, string equipmentDictionaryName, Dictionary<string, List<long>> dummyRuns, Dictionary<long, List<string>> staticRuns, bool useCyclicalForDescription, bool isEAFHosted) :
        base(new Description(), false, smtp, fileParameter, cellInstanceName, connectionCount, cellInstanceConnectionName, fileConnectorConfiguration, equipmentTypeName, parameterizedModelObjectDefinitionType, modelObjectParameters, equipmentDictionaryName, dummyRuns, staticRuns, useCyclicalForDescription, isEAFHosted: connectionCount is null)
    {
        _MinFileLength = 10;
        _NullData = string.Empty;
        _Logistics = new(this);
        if (_FileParameter is null)
            throw new Exception(cellInstanceConnectionName);
        if (_ModelObjectParameterDefinitions is null)
            throw new Exception(cellInstanceConnectionName);
        if (!_IsDuplicator)
            throw new Exception(cellInstanceConnectionName);
        List<string> cssLines = new();
        string cssLinesName = string.Concat("CellInstance.", cellInstanceName, '.', cellInstanceConnectionName, ".CSS.Lines");
        ModelObjectParameterDefinition[] cssLinesDefinitions = GetProperties(cellInstanceConnectionName, modelObjectParameters, cssLinesName);
        foreach (ModelObjectParameterDefinition modelObjectParameterDefinition in cssLinesDefinitions)
            cssLines.Add(modelObjectParameterDefinition.Value);
        _CSSLines = new(cssLines);
        List<string> FrontMatterLines = new();
        string FrontMatterLinesName = string.Concat("CellInstance.", cellInstanceName, '.', cellInstanceConnectionName, ".Front.Matter.Lines");
        ModelObjectParameterDefinition[] FrontMatterLinesDefinitions = GetProperties(cellInstanceConnectionName, modelObjectParameters, FrontMatterLinesName);
        foreach (ModelObjectParameterDefinition modelObjectParameterDefinition in FrontMatterLinesDefinitions)
            FrontMatterLines.Add(modelObjectParameterDefinition.Value);
        _FrontMatterLines = new(FrontMatterLines);
    }

    void IFileRead.Move(Tuple<string, Test[], JsonElement[], List<FileInfo>> extractResults, Exception exception) => Move(extractResults);

    void IFileRead.WaitForThread() => WaitForThread(thread: null, threadExceptions: null);

    string IFileRead.GetEventDescription()
    {
        string result = _Description.GetEventDescription();
        return result;
    }

    List<string> IFileRead.GetHeaderNames()
    {
        List<string> results = _Description.GetHeaderNames();
        return results;
    }

    string[] IFileRead.Move(Tuple<string, Test[], JsonElement[], List<FileInfo>> extractResults, string to, string from, string resolvedFileLocation, Exception exception)
    {
        string[] results = Move(extractResults, to, from, resolvedFileLocation, exception);
        return results;
    }

    JsonProperty[] IFileRead.GetDefault()
    {
        JsonProperty[] results = _Description.GetDefault(this, _Logistics);
        return results;
    }

    Dictionary<string, string> IFileRead.GetDisplayNamesJsonElement()
    {
        Dictionary<string, string> results = _Description.GetDisplayNamesJsonElement(this);
        return results;
    }

    List<IDescription> IFileRead.GetDescriptions(IFileRead fileRead, List<Test> tests, IProcessData processData)
    {
        List<IDescription> results = _Description.GetDescriptions(fileRead, _Logistics, tests, processData);
        return results;
    }

    Tuple<string, Test[], JsonElement[], List<FileInfo>> IFileRead.GetExtractResult(string reportFullPath, string eventName)
    {
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results;
        if (string.IsNullOrEmpty(eventName))
            throw new Exception();
        _ReportFullPath = reportFullPath;
        DateTime dateTime = DateTime.Now;
        results = GetExtractResult(reportFullPath, dateTime);
        if (results.Item3 is null)
            results = new Tuple<string, Test[], JsonElement[], List<FileInfo>>(results.Item1, Array.Empty<Test>(), JsonSerializer.Deserialize<JsonElement[]>("[]"), results.Item4);
        if (results.Item3.Length > 0 && _IsEAFHosted)
            WritePDSF(this, results.Item3);
        UpdateLastTicksDuration(DateTime.Now.Ticks - dateTime.Ticks);
        return results;
    }

    Tuple<string, Test[], JsonElement[], List<FileInfo>> IFileRead.ReExtract()
    {
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results;
        List<string> headerNames = _Description.GetHeaderNames();
        Dictionary<string, string> keyValuePairs = _Description.GetDisplayNamesJsonElement(this);
        results = ReExtract(this, headerNames, keyValuePairs);
        return results;
    }

    private void Save(string key, string fileNameWithoutExtension, string rootDirectory, JsonElement jsonElement)
    {
        string checkFile;
        List<string> indexLines = new();
        indexLines.AddRange(_FrontMatterLines);
        indexLines.Add(string.Empty);
        indexLines.Add($"# {key}");
        indexLines.Add(string.Empty);
        indexLines.Add("## Backlog");
        indexLines.Add(string.Empty);
        indexLines.Add("## Todo");
        indexLines.Add(string.Empty);
        indexLines.Add("## In Progress");
        indexLines.Add(string.Empty);
        indexLines.Add("## Done");
        string singletonDirectory = Path.Combine(rootDirectory, $"{key}-{fileNameWithoutExtension}");
        string vscodeDirectory = Path.Combine(singletonDirectory, ".vscode");
        if (!Directory.Exists(vscodeDirectory))
            _ = Directory.CreateDirectory(vscodeDirectory);
        checkFile = Path.Combine(vscodeDirectory, "settings.json");
        if (!File.Exists(checkFile))
            File.WriteAllText(checkFile, "{ \"[markdown]\": { \"editor.wordWrap\": \"off\" }, \"cSpell.words\": [ \"kanbn\" ] }");
        string kanbanDirectory = Path.Combine(singletonDirectory, ".kanbn");
        if (!Directory.Exists(kanbanDirectory))
            _ = Directory.CreateDirectory(kanbanDirectory);
        checkFile = Path.Combine(kanbanDirectory, ".json");
        if (!File.Exists(checkFile))
            File.WriteAllText(checkFile, jsonElement.ToString());
        else
        {
            string singletonJson = jsonElement.ToString();
            string checkJson = Regex.Replace(File.ReadAllText(checkFile), @"\s+", " ", RegexOptions.Multiline);
            if (Regex.Replace(singletonJson, @"\s+", " ", RegexOptions.Multiline) != checkJson)
                File.WriteAllText(checkFile, singletonJson);
        }
        string tasksDirectory = Path.Combine(kanbanDirectory, "tasks");
        if (!Directory.Exists(tasksDirectory))
            _ = Directory.CreateDirectory(tasksDirectory);
        checkFile = Path.Combine(kanbanDirectory, "board.css");
        if (!File.Exists(checkFile))
            File.WriteAllLines(checkFile, _CSSLines);
        checkFile = Path.Combine(kanbanDirectory, "index.md");
        if (!File.Exists(checkFile))
            File.WriteAllLines(checkFile, indexLines);
    }

    private Tuple<string, Test[], JsonElement[], List<FileInfo>> GetExtractResult(string reportFullPath, DateTime dateTime)
    {
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results;
        string key;
        JsonProperty[] jsonProperties;
        string ticks = dateTime.Ticks.ToString();
        _Logistics = new Logistics(reportFullPath, $"LOGISTICS_1{'\t'}A_JOBID={"BACKLOG"};A_MES_ENTITY={"BACKLOG"};");
        string fileNameWithoutExtension = Path.GetFileNameWithoutExtension(reportFullPath).Replace(" ", "-").Replace("---", "-");
        string rootDirectory = Path.Combine(_FileConnectorConfiguration.SourceFileLocation, fileNameWithoutExtension);
        if (string.IsNullOrEmpty(rootDirectory))
            throw new NullReferenceException(nameof(rootDirectory));
        if (!Directory.Exists(rootDirectory))
            _ = Directory.CreateDirectory(Path.Combine(rootDirectory, ticks));
        string json = File.ReadAllText(reportFullPath);
        JsonElement[] jsonElements = JsonSerializer.Deserialize<JsonElement[]>(json);
        foreach (JsonElement jsonElement in jsonElements)
        {
            if (jsonElement.ValueKind != JsonValueKind.Object)
                continue;
            jsonProperties = jsonElement.EnumerateObject().ToArray();
            if (jsonProperties.Length < 2)
                continue;
            key = jsonProperties[0].Value.ToString();
            if (string.IsNullOrEmpty(key))
                continue;
            if (string.IsNullOrEmpty(jsonProperties[1].Value.ToString()))
                continue;
            Save(key, fileNameWithoutExtension, rootDirectory, jsonElement);
            if (!_IsEAFHosted)
                break;
        }
        results = new(_Logistics.Logistics1[0], Array.Empty<Test>(), Array.Empty<JsonElement>(), new List<FileInfo>());
        return results;
    }

}